﻿using System;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Xml.Linq;

namespace VeloToulouse.Entites
{
    public class Station:INotifyPropertyChanged
    {

        private string _id;
        public string Id
        {
            get { return _id; }
            set
            {
                if (value != _id)
                {
                    _id = value;
                    NotifyPropertyChanged("Id");
                }
            }
        }

        private string _nom;
        public string Nom
        {
            get { return _nom; }
            set
            {
                if (value != _nom)
                {
                    _nom = value;
                    NotifyPropertyChanged("Nom");
                }
            }
        }

        private string _adresse;
        public string Adresse
        {
            get
            {
                return _adresse;
            }
            set
            {
                if (value != _adresse)
                {
                    _adresse = value;
                    NotifyPropertyChanged("Adresse");
                }
            }
        }
        
        private string _latitude;
        public string Latitude
        {
            get
            {
                return _latitude;
            }
            set
            {
                if (value != _latitude)
                {
                    _latitude = value;
                    NotifyPropertyChanged("Latitude");
                }
            }
        }
   
        private string _longitude;
        public string Longitude
        {
            get
            {
                return _longitude;
            }
            set
            {
                if (value != _longitude)
                {
                    _longitude = value;
                    NotifyPropertyChanged("Longitude");
                }
            }
        }

        private int _VelosDisponibles;
        public int VelosDisponibles
        {
            get
            {
                return _VelosDisponibles;
            }
            set
            {
                if (value != _VelosDisponibles)
                {
                    _VelosDisponibles = value;
                    NotifyPropertyChanged("VelosDisponibles");
                }
            }
        }
        
        private int _emplacements;
        public int Emplacements
        {
            get
            {
                return _emplacements;
            }
            set
            {
                if (value != _emplacements)
                {
                    _emplacements = value;
                    NotifyPropertyChanged("Emplacements");
                }
            }
        }

        public Uri MapImageUrl
        {
            get
            {
                return new Uri(string.Format("http://maps.google.com/maps/api/staticmap?center={0},{1}&zoom=16&size=400x400&maptype=roadmap&sensor=false&markers=color:green|{0},{1}&key={2}", Latitude,Longitude,gmapsApiKey));
            }
        }

        private bool _stationFavorite;
        public bool StationFavorite
        {
            get
            {
                return _stationFavorite;
            }
            set
            {
                if (value != _stationFavorite)
                {
                    _stationFavorite = value;
                    NotifyPropertyChanged("StationFavorite");
                }
            }
        }

        public void RefreshAsync()
        {
            // TODO : Rajouter le code pour VelosDispos, Points, Emplacements.
            WebClient webClient = new WebClient();
            webClient.OpenReadCompleted += (sender, e) =>
            {
                if (e.Error != null)
                    return;
                Stream s = e.Result;
                XDocument xdoc = XDocument.Load(s);

                this.VelosDisponibles = Convert.ToInt32(xdoc.Descendants("available").First().Value);
                this.Emplacements = Convert.ToInt32(xdoc.Descendants("total").First().Value);
            };
            webClient.OpenReadAsync(new Uri(_serviceUrl+Id));
        }

        private static readonly string gmapsApiKey = "ABQIAAAA08mpz4RaTnz7hCbpALvTmxSeHeCpENPAfyL5OD2fjeJHj29tbxQtcO6XTFwXIZ1bTltyI_a-sga2VQ";
        private static readonly string _serviceUrl = "http://www.velo.toulouse.fr/service/stationdetails/";
        
        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
