﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Xml.Linq;
using Microsoft.Phone.Shell;
using VeloToulouse.Code;
using VeloToulouse.Entites;

namespace VeloToulouse
{
    public class StationsService
    {
        private const string StationUrl = "http://www.velo.toulouse.fr/service/carto";
        public ObservableCollection<Station> Stations{ get; set;}
        public ObservableCollection<Station> StationsFavorites { get; set; }
        
        private static StationsService _current;

        public static StationsService Current
        {
            get { if(_current == null)
                _current = new StationsService();
                return _current;
            }
            private set { _current = value; }
        }

        private StationsService()
        {
            Stations = new ObservableCollection<Station>();
            StationsFavorites = new ObservableCollection<Station>();
            PhoneApplicationService.Current.Deactivated += new EventHandler<DeactivatedEventArgs>(Current_Deactivated);
            

        }

        public void WakeUpService()
{
     Stations = (ObservableCollection<Station>) PhoneApplicationService.Current.State["Stations"] ;
     StationsFavorites =(ObservableCollection<Station>) PhoneApplicationService.Current.State["StationsFavorites"];

} 
        void Current_Deactivated(object sender, DeactivatedEventArgs e)
        {
            PhoneApplicationService.Current.State["Stations"] = Stations;
            PhoneApplicationService.Current.State["StationsFavorites"] = StationsFavorites;
            
        }

        public Station GetStationById(string id)
        {
            return (from s in this.Stations
                   where s.Id.Equals(id)
                   select s).FirstOrDefault();
        }

        public void GetListeStationsAsync()
        {
            var webClient = new WebClient();
            
            webClient.OpenReadCompleted += LoadCompleted;
            webClient.OpenReadAsync(new Uri(StationUrl));
        }

        private void LoadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                InvokeGetListeStationsCompleted(null);
                return;
            }
                
            Stream s = e.Result;
            XDocument xdoc = XDocument.Load(s);

            List<Station> res = (xdoc.Descendants("marker").Select(
                                        i => new Station{
                                            Id = i.Attribute("number").Value,
                                            Nom = i.Attribute("name").Value, 
                                            Adresse = i.Attribute("fullAddress").Value, 
                                            Latitude = i.Attribute("lat").Value, 
                                            Longitude = i.Attribute("lng").Value
                                            })).ToList();
            
            res.ForEach(item => Stations.Add(item));
            LoadFavoris();
            InvokeGetListeStationsCompleted(new EventArgs());
        }

        public event EventHandler GetListeStationsCompleted;

        public void InvokeGetListeStationsCompleted(EventArgs e)
        {
            EventHandler handler = GetListeStationsCompleted;
            if (handler != null) handler(this, e);
        }

        public void LoadFavoris()
        {
            if (!IsolatedStorageSettings.ApplicationSettings.Contains("favoris"))
                IsolatedStorageSettings.ApplicationSettings["favoris"] = new ObservableCollection<Station>();

            /// A chaque chargement de l'application, nous récupérons de nouveaux objets stations,
            /// même si les IDs correspondent, nous avons deux objets différents. Il faut donc reconstruire la
            /// liste des favoris
            ObservableCollection<Station> sf = (ObservableCollection<Station>)IsolatedStorageSettings.ApplicationSettings["favoris"];

            List<Station> temp =  (from s in Stations
                                                        where sf.Contains(s, new LambdaComparer<Station>((x, y) => x.Id == y.Id))
                                                     select s).ToList();
            temp.ForEach(t => {StationsFavorites.Add(t);
                                  t.StationFavorite = true;})   ;
        System.Diagnostics.Debug.WriteLine("coll");
            
        }
        public void SaveFavoris()
        {
            IsolatedStorageSettings.ApplicationSettings["favoris"] = StationsFavorites;
            IsolatedStorageSettings.ApplicationSettings.Save();
        }
    }
}